# Woclema - Weblob, Originally Command Line Editable/Manageable
# Copyright (C) 2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package Woclema::Calendar;

use strict;
use warnings FATAL => 'all';
use diagnostics;


my $dsec = 24 * 60 * 60;


=item get_cont_month_limits
Get date boundaries for month of given day as they'd be on a continuous
calendar.  Will probably get broken on days when clocks are adjusted for
daylight saving.

0: timestamp (from epoch)
1: first day of week
r: (first from epoch, last from epoch)
=cut
sub get_cont_month_limits($$) {
	my ($given_epoch, $fdow) = @_;
	my $adjust;
	$fdow = 1 unless defined($fdow);

	my @given_date = gmtime($given_epoch);

	# Find first day
	my $first_epoch = $given_epoch - ($given_date[3] - 1) * $dsec;
	my @first_date = gmtime($first_epoch);
	$adjust = $fdow - $first_date[6];
	$adjust -= 7 if ($adjust > 0);
	$first_epoch += $adjust * $dsec;
	$first_epoch -= (($first_date[2] * 60) + $first_date[1]) * 60
		+ $first_date[0];

	# Find last day -- horrible kludge to avoid leap-year calculation
	my $given_month = $given_date[4];
	my ($last_epoch, @last_date);
	for (my $i = 0; $i < 4; $i++) {
		$last_epoch = $given_epoch + (31 - $given_date[3] - $i) * $dsec;
		@last_date = gmtime($last_epoch);
		last if ($last_date[4] == $given_month);
	}
	$adjust = (6 - ($last_date[6] - $fdow)) % 7;
	$last_epoch += $adjust * $dsec;
	$last_epoch += ((23 - $last_date[2]) * 60
			+ (59 - $last_date[1])) * 60
		+ 59 - $last_date[0];

	return ($first_epoch, $last_epoch);
}


=item get_month_limits
Get date boundaries for month of given day.
Will probably get broken on days when clocks are adjusted for daylight saving.

0: timestamp (from epoch)
1: first day of week
r: (first from epoch, last from epoch)
=cut
sub get_month_limits($$) {
	my ($given_epoch, $fdow) = @_;

	my @given_date = gmtime($given_epoch);

	# Find first day
	my $first_epoch = $given_epoch - ($given_date[3] - 1) * $dsec;
	my @first_date = gmtime($first_epoch);
	$first_epoch -= (($first_date[2] * 60) + $first_date[1]) * 60
		+ $first_date[0];

	# Find last day -- horrible kludge to avoid leap-year calculation
	my $given_month = $given_date[4];
	my ($last_epoch, @last_date);
	for (my $i = 0; $i < 4; $i++) {
		$last_epoch = $given_epoch + (31 - $given_date[3] - $i) * $dsec;
		@last_date = gmtime($last_epoch);
		last if ($last_date[4] == $given_month);
	}
	$last_epoch += ((23 - $last_date[2]) * 60
			+ (59 - $last_date[1])) * 60
		+ 59 - $last_date[0];

	return ($first_epoch, $last_epoch);
}


=item get_day_limits
Get date boundaries for given day.

0: timestamp (UNIX epoch)
r: (first, last)
=cut
sub get_day_limits($$) {
	my ($stamp) = @_;

	my @date = gmtime($stamp);

	my $a = (($date[2] * 60) + $date[1]) * 60 + $date[0];
	my $b = ((23 - $date[2]) * 60 + (59 - $date[1])) * 60 + 59 - $date[0];

	return ($stamp - $a, $stamp + $b);
}


=item day_name
Get day name.

0: day number (0 - 6, 0 = Sunday)
r: day name
=cut
sub day_name($) {
	my ($no) = @_;
	my @day = ( "Sunday", "Monday", "Tuesday", "Wednesday",
		"Thursday", "Friday", "Sunday" );
	return ($no >= 0 && $no <= 6) ? $day[$no] : "?$no?";
}


=item day_short
Get short for day.

0: day number (0 - 6, 0 = Sunday)
r: day name
=cut
sub day_short($) {
	my ($no) = @_;
	my @day = ( "Su", "Mo", "Tu", "We", "Th", "Fr", "Sa" );
	return ($no >= 0 && $no <= 6) ? $day[$no] : "?$no?";
}


=item month_name
Get month name.

0: month number (1 - 12)
r: month name
=cut
sub month_name($) {
	my ($no) = @_;
	my @month = ( "January", "February", "March", "April",
			"May", "June", "July", "August",
			"September", "October", "November", "December" );
	return ($no >= 1 && $no <= 12) ? $month[($no - 1)] : "?$no?";
}


=item month_short
Get short for month.

0: month number (1 - 12)
r: month name
=cut
sub month_short($) {
	my ($no) = @_;
	my @month = ( "Jan", "Feb", "Mar", "Apr", "May", "Jun",
			"Jul", "Aug", "Sep", "Oct", "Nov", "Dec" );
	return ($no >= 1 && $no <= 12) ? $month[($no - 1)] : "?$no?";
}


return 1;
